@@ -19,28 +19,20 @@
 Base attribute driver class
 """
 
-import logging
 import os.path
 
 from glance.common import exception
 from glance.common import utils
+from glance.openstack.common import cfg
+import glance.openstack.common.log as logging
 
+LOG = logging.getLogger(__name__)
 
-logger = logging.getLogger(__name__)
+CONF = cfg.CONF
 
 
 class Driver(object):
 
-    def __init__(self, conf):
-        """
-        Initialize the attribute driver with a set of options.
-
-        :param conf: Dictionary of configuration options
-        :raises `exception.BadDriverConfiguration` if configuration of the
-                driver fails for any reason.
-        """
-        self.conf = conf or {}
-
     def configure(self):
         """
         Configure the driver to use the stored configuration options
@@ -58,10 +50,10 @@ class Driver(object):
         Creates all necessary directories under the base cache directory
         """
 
-        self.base_dir = self.conf.image_cache_dir
+        self.base_dir = CONF.image_cache_dir
         if self.base_dir is None:
             msg = _('Failed to read %s from config') % 'image_cache_dir'
-            logger.error(msg)
+            LOG.error(msg)
             driver = self.__class__.__module__
             raise exception.BadDriverConfiguration(driver_name=driver,
                                                    reason=msg)
