@@ -1,4 +1,6 @@
-# Copyright 2010-2011 OpenStack Foundation
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
+# Copyright 2010-2011 OpenStack LLC.
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -13,15 +15,13 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from oslo_log import log as logging
-from oslo_utils import encodeutils
 import webob.exc
 
 from glance.common import exception
 from glance.common import utils
 from glance.common import wsgi
 import glance.db
-from glance.i18n import _, _LI, _LW
+import glance.openstack.common.log as logging
 
 
 LOG = logging.getLogger(__name__)
@@ -35,49 +35,28 @@ class Controller(object):
 
     def __init__(self):
         self.db_api = glance.db.get_api()
-
-    def is_image_sharable(self, context, image):
-        """Return True if the image can be shared to others in this context."""
-        # Is admin == image sharable
-        if context.is_admin:
-            return True
-
-        # Only allow sharing if we have an owner
-        if context.owner is None:
-            return False
-
-        # If we own the image, we can share it
-        if context.owner == image['owner']:
-            return True
-
-        members = self.db_api.image_member_find(context,
-                                                image_id=image['id'],
-                                                member=context.owner)
-        if members:
-            return members[0]['can_share']
-
-        return False
+        self.db_api.configure_db()
 
     def index(self, req, image_id):
         """
         Get the members of an image.
         """
         try:
-            self.db_api.image_get(req.context, image_id, v1_mode=True)
+            self.db_api.image_get(req.context, image_id)
         except exception.NotFound:
-            msg = _("Image %(id)s not found") % {'id': image_id}
-            LOG.warn(msg)
-            raise webob.exc.HTTPNotFound(msg)
+            msg = _("Image %(id)s not found")
+            LOG.info(msg % {'id': image_id})
+            raise webob.exc.HTTPNotFound()
         except exception.Forbidden:
             # If it's private and doesn't belong to them, don't let on
             # that it exists
-            msg = _LW("Access denied to image %(id)s but returning"
-                      " 'not found'") % {'id': image_id}
-            LOG.warn(msg)
+            msg = _("Access denied to image %(id)s but returning 'not found'")
+            LOG.info(msg % {'id': image_id})
             raise webob.exc.HTTPNotFound()
 
         members = self.db_api.image_member_find(req.context, image_id=image_id)
-        LOG.debug("Returning member list for image %(id)s", {'id': image_id})
+        msg = _("Returning member list for image %(id)s")
+        LOG.info(msg % {'id': image_id})
         return dict(members=make_member_list(members,
                                              member_id='member',
                                              can_share='can_share'))
@@ -88,46 +67,45 @@ class Controller(object):
         Replaces the members of the image with those specified in the
         body.  The body is a dict with the following format::
 
-            {'memberships': [
-                {'member_id': <MEMBER_ID>,
-                 ['can_share': [True|False]]}, ...
+            {"memberships": [
+                {"member_id": <MEMBER_ID>,
+                 ["can_share": [True|False]]}, ...
             ]}
         """
         self._check_can_access_image_members(req.context)
 
         # Make sure the image exists
+        session = self.db_api.get_session()
         try:
-            image = self.db_api.image_get(req.context, image_id, v1_mode=True)
+            image = self.db_api.image_get(req.context, image_id,
+                                          session=session)
         except exception.NotFound:
-            msg = _("Image %(id)s not found") % {'id': image_id}
-            LOG.warn(msg)
-            raise webob.exc.HTTPNotFound(msg)
+            msg = _("Image %(id)s not found")
+            LOG.info(msg % {'id': image_id})
+            raise webob.exc.HTTPNotFound()
         except exception.Forbidden:
             # If it's private and doesn't belong to them, don't let on
             # that it exists
-            msg = _LW("Access denied to image %(id)s but returning"
-                      " 'not found'") % {'id': image_id}
-            LOG.warn(msg)
+            msg = _("Access denied to image %(id)s but returning 'not found'")
+            LOG.info(msg % {'id': image_id})
             raise webob.exc.HTTPNotFound()
 
         # Can they manipulate the membership?
-        if not self.is_image_sharable(req.context, image):
-            msg = (_LW("User lacks permission to share image %(id)s") %
-                   {'id': image_id})
-            LOG.warn(msg)
+        if not self.db_api.is_image_sharable(req.context, image):
+            msg = _("User lacks permission to share image %(id)s")
+            LOG.info(msg % {'id': image_id})
             msg = _("No permission to share that image")
             raise webob.exc.HTTPForbidden(msg)
 
         # Get the membership list
         try:
             memb_list = body['memberships']
-        except Exception as e:
+        except Exception, e:
             # Malformed entity...
-            msg = _LW("Invalid membership association specified for "
-                      "image %(id)s") % {'id': image_id}
-            LOG.warn(msg)
-            msg = (_("Invalid membership association: %s") %
-                   encodeutils.exception_to_unicode(e))
+            msg = _("Invalid membership association specified for "
+                    "image %(id)s")
+            LOG.info(msg % {'id': image_id})
+            msg = _("Invalid membership association: %s") % e
             raise webob.exc.HTTPBadRequest(explanation=msg)
 
         add = []
@@ -138,13 +116,12 @@ class Controller(object):
                 datum = dict(image_id=image['id'],
                              member=memb['member_id'],
                              can_share=None)
-            except Exception as e:
+            except Exception, e:
                 # Malformed entity...
-                msg = _LW("Invalid membership association specified for "
-                          "image %(id)s") % {'id': image_id}
-                LOG.warn(msg)
-                msg = (_("Invalid membership association: %s") %
-                       encodeutils.exception_to_unicode(e))
+                msg = _("Invalid membership association specified for "
+                        "image %(id)s")
+                LOG.info(msg % {'id': image_id})
+                msg = _("Invalid membership association: %s") % e
                 raise webob.exc.HTTPBadRequest(explanation=msg)
 
             # Figure out what can_share should be
@@ -155,7 +132,7 @@ class Controller(object):
             members = self.db_api.image_member_find(req.context,
                                                     image_id=datum['image_id'],
                                                     member=datum['member'],
-                                                    include_deleted=True)
+                                                    session=session)
             try:
                 member = members[0]
             except IndexError:
@@ -176,27 +153,25 @@ class Controller(object):
         # memberships to modify.  Let's start by walking through all
         # the existing image memberships...
         existing_members = self.db_api.image_member_find(req.context,
-                                                         image_id=image['id'],
-                                                         include_deleted=True)
-        for member in existing_members:
-            if member['id'] in existing:
+                                                         image_id=image['id'])
+        for memb in existing_members:
+            if memb['id'] in existing:
                 # Just update the membership in place
-                update = existing[member['id']]['values']
-                self.db_api.image_member_update(req.context,
-                                                member['id'],
-                                                update)
+                update = existing[memb['id']]['values']
+                self.db_api.image_member_update(req.context, memb, update,
+                                                session=session)
             else:
-                if not member['deleted']:
-                    # Outdated one; needs to be deleted
-                    self.db_api.image_member_delete(req.context, member['id'])
+                # Outdated one; needs to be deleted
+                self.db_api.image_member_delete(req.context, memb,
+                                                session=session)
 
         # Now add the non-existent ones
         for memb in add:
-            self.db_api.image_member_create(req.context, memb)
+            self.db_api.image_member_create(req.context, memb, session=session)
 
         # Make an appropriate result
-        LOG.info(_LI("Successfully updated memberships for image %(id)s"),
-                 {'id': image_id})
+        msg = _("Successfully updated memberships for image %(id)s")
+        LOG.info(msg % {'id': image_id})
         return webob.exc.HTTPNoContent()
 
     @utils.mutating
@@ -205,11 +180,11 @@ class Controller(object):
         Adds a membership to the image, or updates an existing one.
         If a body is present, it is a dict with the following format::
 
-            {'member': {
-                'can_share': [True|False]
+            {"member": {
+                "can_share": [True|False]
             }}
 
-        If `can_share` is provided, the member's ability to share is
+        If "can_share" is provided, the member's ability to share is
         set accordingly.  If it is not provided, existing memberships
         remain unchanged and new memberships default to False.
         """
@@ -217,24 +192,22 @@ class Controller(object):
 
         # Make sure the image exists
         try:
-            image = self.db_api.image_get(req.context, image_id, v1_mode=True)
+            image = self.db_api.image_get(req.context, image_id)
         except exception.NotFound:
-            msg = _("Image %(id)s not found") % {'id': image_id}
-            LOG.warn(msg)
-            raise webob.exc.HTTPNotFound(msg)
+            msg = _("Image %(id)s not found")
+            LOG.info(msg % {'id': image_id})
+            raise webob.exc.HTTPNotFound()
         except exception.Forbidden:
             # If it's private and doesn't belong to them, don't let on
             # that it exists
-            msg = _LW("Access denied to image %(id)s but returning"
-                      " 'not found'") % {'id': image_id}
-            LOG.warn(msg)
+            msg = _("Access denied to image %(id)s but returning 'not found'")
+            LOG.info(msg % {'id': image_id})
             raise webob.exc.HTTPNotFound()
 
         # Can they manipulate the membership?
-        if not self.is_image_sharable(req.context, image):
-            msg = (_LW("User lacks permission to share image %(id)s") %
-                   {'id': image_id})
-            LOG.warn(msg)
+        if not self.db_api.is_image_sharable(req.context, image):
+            msg = _("User lacks permission to share image %(id)s")
+            LOG.info(msg % {'id': image_id})
             msg = _("No permission to share that image")
             raise webob.exc.HTTPForbidden(msg)
 
@@ -243,33 +216,33 @@ class Controller(object):
         if body:
             try:
                 can_share = bool(body['member']['can_share'])
-            except Exception as e:
+            except Exception, e:
                 # Malformed entity...
-                msg = _LW("Invalid membership association specified for "
-                          "image %(id)s") % {'id': image_id}
-                LOG.warn(msg)
-                msg = (_("Invalid membership association: %s") %
-                       encodeutils.exception_to_unicode(e))
+                msg = _("Invalid membership association specified for "
+                        "image %(id)s")
+                LOG.info(msg % {'id': image_id})
+                msg = _("Invalid membership association: %s") % e
                 raise webob.exc.HTTPBadRequest(explanation=msg)
 
         # Look up an existing membership...
+        session = self.db_api.get_session()
         members = self.db_api.image_member_find(req.context,
                                                 image_id=image_id,
                                                 member=id,
-                                                include_deleted=True)
+                                                session=session)
         if members:
             if can_share is not None:
                 values = dict(can_share=can_share)
-                self.db_api.image_member_update(req.context,
-                                                members[0]['id'],
-                                                values)
+                self.db_api.image_member_update(req.context, members[0],
+                                                values, session=session)
         else:
             values = dict(image_id=image['id'], member=id,
                           can_share=bool(can_share))
-            self.db_api.image_member_create(req.context, values)
+            self.db_api.image_member_create(req.context, values,
+                                            session=session)
 
-        LOG.info(_LI("Successfully updated a membership for image %(id)s"),
-                 {'id': image_id})
+        msg = _("Successfully updated a membership for image %(id)s")
+        LOG.info(msg % {'id': image_id})
         return webob.exc.HTTPNoContent()
 
     @utils.mutating
@@ -281,65 +254,57 @@ class Controller(object):
 
         # Make sure the image exists
         try:
-            image = self.db_api.image_get(req.context, image_id, v1_mode=True)
+            image = self.db_api.image_get(req.context, image_id)
         except exception.NotFound:
-            msg = _("Image %(id)s not found") % {'id': image_id}
-            LOG.warn(msg)
-            raise webob.exc.HTTPNotFound(msg)
+            msg = _("Image %(id)s not found")
+            LOG.info(msg % {'id': image_id})
+            raise webob.exc.HTTPNotFound()
         except exception.Forbidden:
             # If it's private and doesn't belong to them, don't let on
             # that it exists
-            msg = _LW("Access denied to image %(id)s but returning"
-                      " 'not found'") % {'id': image_id}
-            LOG.warn(msg)
+            msg = _("Access denied to image %(id)s but returning 'not found'")
+            LOG.info(msg % {'id': image_id})
             raise webob.exc.HTTPNotFound()
 
         # Can they manipulate the membership?
-        if not self.is_image_sharable(req.context, image):
-            msg = (_LW("User lacks permission to share image %(id)s") %
-                   {'id': image_id})
-            LOG.warn(msg)
+        if not self.db_api.is_image_sharable(req.context, image):
+            msg = _("User lacks permission to share image %(id)s")
+            LOG.info(msg % {'id': image_id})
             msg = _("No permission to share that image")
             raise webob.exc.HTTPForbidden(msg)
 
         # Look up an existing membership
-        members = self.db_api.image_member_find(req.context,
-                                                image_id=image_id,
-                                                member=id)
-        if members:
-            self.db_api.image_member_delete(req.context, members[0]['id'])
-        else:
-            LOG.debug("%(id)s is not a member of image %(image_id)s",
-                      {'id': id, 'image_id': image_id})
-            msg = _("Membership could not be found.")
-            raise webob.exc.HTTPNotFound(explanation=msg)
+        try:
+            session = self.db_api.get_session()
+            members = self.db_api.image_member_find(req.context,
+                                                    image_id=image_id,
+                                                    member=id,
+                                                    session=session)
+            self.db_api.image_member_delete(req.context,
+                                            members[0],
+                                            session=session)
+        except exception.NotFound:
+            pass
 
         # Make an appropriate result
-        LOG.info(_LI("Successfully deleted a membership from image %(id)s"),
-                 {'id': image_id})
+        msg = _("Successfully deleted a membership from image %(id)s")
+        LOG.info(msg % {'id': image_id})
         return webob.exc.HTTPNoContent()
 
-    def default(self, req, *args, **kwargs):
-        """This will cover the missing 'show' and 'create' actions"""
-        LOG.debug("The method %s is not allowed for this resource",
-                  req.environ['REQUEST_METHOD'])
-        raise webob.exc.HTTPMethodNotAllowed(
-            headers=[('Allow', 'PUT, DELETE')])
-
     def index_shared_images(self, req, id):
         """
         Retrieves images shared with the given member.
         """
         try:
             members = self.db_api.image_member_find(req.context, member=id)
-        except exception.NotFound:
-            msg = _LW("Member %(id)s not found") % {'id': id}
-            LOG.warn(msg)
+        except exception.NotFound, e:
+            msg = _("Member %(id)s not found")
+            LOG.info(msg % {'id': id})
             msg = _("Membership could not be found.")
             raise webob.exc.HTTPBadRequest(explanation=msg)
 
-        LOG.debug("Returning list of images shared with member %(id)s",
-                  {'id': id})
+        msg = _("Returning list of images shared with member %(id)s")
+        LOG.info(msg % {'id': id})
         return dict(shared_images=make_member_list(members,
                                                    image_id='image_id',
                                                    can_share='can_share'))
@@ -353,10 +318,12 @@ def make_member_list(members, **attr_map):
     """
 
     def _fetch_memb(memb, attr_map):
-        return {k: memb[v] for k, v in attr_map.items() if v in memb.keys()}
+        return dict([(k, memb[v]) for k, v in attr_map.items()
+                                  if v in memb.keys()])
 
     # Return the list of members with the given attribute mapping
-    return [_fetch_memb(memb, attr_map) for memb in members]
+    return [_fetch_memb(memb, attr_map) for memb in members
+            if not memb.deleted]
 
 
 def create_resource():
