@@ -17,48 +17,21 @@
 
 """Functional test case that utilizes the bin/glance CLI tool"""
 
+import BaseHTTPServer
 import datetime
 import httplib2
 import json
 import os
 import tempfile
+import thread
+import time
 
-from glance.common import utils
+from glance.openstack.common import timeutils
 from glance.tests import functional
-from glance.tests.utils import execute, requires, minimal_add_command
 from glance.tests.functional.store_utils import (setup_http,
                                                  teardown_http,
                                                  get_http_uri)
-
-
-class TestBinGlanceAuth(functional.FunctionalTest):
-    """Functional tests for bin/glance with some amount of auth"""
-
-    def assertIn(self, key, bag):
-        msg = 'Expected to find substring "%s" in "%s"' % (key, bag)
-        self.assertTrue(key in bag, msg)
-
-    def assertNotIn(self, key, bag):
-        msg = 'Expected not to find substring "%s" in "%s"' % (key, bag)
-        self.assertFalse(key in bag, msg)
-
-    def test_index_with_https_auth(self):
-        self.cleanup()
-        self.start_servers(**self.__dict__.copy())
-
-        api_port = self.api_port
-        cmd = ("bin/glance --port=%d -N https://this.url.doesnt.matter/ "
-               "-A aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa index") % api_port
-        exitcode, out, err = execute(cmd, raise_error=False)
-
-        #NOTE(markwash): we should expect the command to fail because the
-        # testing glance api server is not configured to authenticate, and the
-        # token we provide is invalid. However, it should fail due to
-        # NotAuthenticated, rather than because of an SSL error.
-
-        self.assertNotEqual(0, exitcode)
-        self.assertNotIn('SSL23_GET_SERVER_HELLO', out)
-        self.assertIn('NotAuthenticated: You are not authenticated.', out)
+from glance.tests.utils import execute, requires, minimal_add_command
 
 
 class TestBinGlance(functional.FunctionalTest):
@@ -72,11 +45,28 @@ class TestBinGlance(functional.FunctionalTest):
         # spin up won't have keystone support, so we need to switch to the
         # NoAuth strategy.
         os.environ['OS_AUTH_STRATEGY'] = 'noauth'
+        os.environ['OS_AUTH_URL'] = ''
 
     def _assertStartsWith(self, str, prefix):
         msg = 'expected "%s" to start with "%s"' % (str, prefix)
         self.assertTrue(str.startswith(prefix), msg)
 
+    def _assertNotIn(self, key, bag):
+        msg = 'Expected not to find substring "%s" in "%s"' % (key, bag)
+        self.assertFalse(key in bag, msg)
+
+    def test_index_with_https(self):
+        self.cleanup()
+        self.start_servers(**self.__dict__.copy())
+
+        cmd = ("bin/glance -N https://auth/ --port=%d index") % self.api_port
+        exitcode, out, err = execute(cmd, raise_error=False)
+
+        self.assertNotEqual(0, exitcode)
+        self._assertNotIn('SSL23_GET_SERVER_HELLO', out)
+
+        self.stop_servers()
+
     def test_add_with_location_and_id(self):
         self.cleanup()
         self.start_servers(**self.__dict__.copy())
@@ -133,6 +123,8 @@ class TestBinGlance(functional.FunctionalTest):
         self.assertEqual('0', size, "Expected image to be 0 bytes in size, "
                                     "but got %s. " % size)
 
+        self.stop_servers()
+
     def test_add_with_location(self):
         self.cleanup()
         self.start_servers(**self.__dict__.copy())
@@ -151,7 +143,7 @@ class TestBinGlance(functional.FunctionalTest):
         # 1. Add public image
         cmd = minimal_add_command(api_port,
                                   'MyImage',
-                                  'location=http://example.com')
+                                  'location=http://localhost:0')
         exitcode, out, err = execute(cmd)
 
         self.assertEqual(0, exitcode)
@@ -168,65 +160,14 @@ class TestBinGlance(functional.FunctionalTest):
 
         line = lines[0]
 
-        image_id, name, disk_format, container_format, size = \
-            [c.strip() for c in line.split()]
+        img_info = [c.strip() for c in line.split()]
+        image_id, name, disk_format, container_format, size = img_info
         self.assertEqual('MyImage', name)
 
         self.assertEqual('0', size, "Expected image to be 0 bytes in size, "
                                     "but got %s. " % size)
 
-    def _verify_owner(self, owner, image_id):
-        cmd = "bin/glance --port=%d show %s" % (self.api_port, image_id)
-        exitcode, out, err = execute(cmd)
-        self.assertEqual(0, exitcode)
-
-        # verify expected owner as first class attribute
-        self.assertTrue(('Owner: %s' % owner) in out)
-        # ensure owner does not appear as a custom property
-        self.assertFalse("Property 'owner':" in out)
-
-    def test_add_with_owner_admin(self):
-        """Test setting ownership of new image by admin user"""
-        self.cleanup()
-        self.start_servers(**self.__dict__.copy())
-
-        # ownership set by admin user (defaults as such due to no-auth)
-        cmd = minimal_add_command(self.api_port,
-                                  'MyImage',
-                                  '--silent-upload owner=42')
-        exitcode, out, err = execute(cmd)
-
-        self.assertEqual(0, exitcode)
-        self.assertTrue(out.strip().startswith('Added new image with ID:'))
-
-        image_id = out.strip().replace('Added new image with ID: ', '')
-
-        self._verify_owner('42', image_id)
-
-    def test_add_with_owner_non_admin(self):
-        """Test setting ownership of new image by non-admin user"""
-        self.cleanup()
-        self.api_server.deployment_flavor = 'fakeauth'
-        self.registry_server.deployment_flavor = 'fakeauth'
-        self.start_servers(**self.__dict__.copy())
-
-        # ownership set by non-admin user (setup as such by fakeauth pipeline)
-        headers = {'X-Image-Meta-Name': 'MyImage',
-                   'X-Image-Meta-disk_format': 'raw',
-                   'X-Image-Meta-container_format': 'ovf',
-                   'X-Image-Meta-Is-Public': 'True',
-                   'X-Image-Meta-Owner': '42',
-                   'X-Auth-Token': 'Confirmed:pattieblack:froggy:demo',
-        }
-
-        path = "http://%s:%d/v1/images" % ("0.0.0.0", self.api_port)
-        http = httplib2.Http()
-        response, content = http.request(path, 'POST', headers=headers)
-        self.assertEqual(response.status, 201)
-        data = json.loads(content)
-        image_id = data['image']['id']
-
-        self._verify_owner('froggy', image_id)
+        self.stop_servers()
 
     def test_add_no_name(self):
         self.cleanup()
@@ -248,7 +189,7 @@ class TestBinGlance(functional.FunctionalTest):
         # name...
         cmd = ("bin/glance --port=%d add is_public=True"
                " disk_format=raw container_format=ovf"
-               " %s" % (api_port, 'location=http://example.com'))
+               " %s" % (api_port, 'location=http://localhost:0'))
         exitcode, out, err = execute(cmd)
 
         self.assertEqual(0, exitcode)
@@ -269,11 +210,15 @@ class TestBinGlance(functional.FunctionalTest):
             [c.strip() for c in line.split()]
         self.assertEqual('None', name)
 
-    @requires(setup_http, teardown_http)
+        self.stop_servers()
+
+    @requires(teardown=teardown_http)
     def test_add_copying_from(self):
         self.cleanup()
         self.start_servers(**self.__dict__.copy())
 
+        setup_http(self)
+
         api_port = self.api_port
         registry_port = self.registry_port
 
@@ -311,10 +256,14 @@ class TestBinGlance(functional.FunctionalTest):
         self.assertEqual('5120', size, "Expected image to be 5120 bytes "
                                        " in size, but got %s. " % size)
 
+        self.stop_servers()
+
     def _do_test_update_external_source(self, source):
         self.cleanup()
         self.start_servers(**self.__dict__.copy())
 
+        setup_http(self)
+
         api_port = self.api_port
         registry_port = self.registry_port
 
@@ -361,7 +310,9 @@ class TestBinGlance(functional.FunctionalTest):
         lines = out.split("\n")
         self.assertTrue(set(lines) >= set(expected_lines))
 
-    @requires(setup_http, teardown_http)
+        self.stop_servers()
+
+    @requires(teardown=teardown_http)
     def test_update_copying_from(self):
         """
         Tests creating an queued image then subsequently updating
@@ -369,7 +320,7 @@ class TestBinGlance(functional.FunctionalTest):
         """
         self._do_test_update_external_source('copy_from')
 
-    @requires(setup_http, teardown_http)
+    @requires(teardown=teardown_http)
     def test_update_location(self):
         """
         Tests creating an queued image then subsequently updating
@@ -399,7 +350,7 @@ class TestBinGlance(functional.FunctionalTest):
             file_name = image_file.name
             cmd = minimal_add_command(api_port,
                                      'MyImage',
-                                     'location=http://example.com < %s' %
+                                     'location=http://localhost:0 < %s' %
                                      file_name)
             exitcode, out, err = execute(cmd)
 
@@ -417,13 +368,15 @@ class TestBinGlance(functional.FunctionalTest):
 
         line = lines[0]
 
-        image_id, name, disk_format, container_format, size = \
-            [c.strip() for c in line.split()]
+        img_info = [c.strip() for c in line.split()]
+        image_id, name, disk_format, container_format, size = img_info
         self.assertEqual('MyImage', name)
 
         self.assertEqual('0', size, "Expected image to be 0 bytes in size, "
                                     "but got %s. " % size)
 
+        self.stop_servers()
+
     def test_add_list_delete_list(self):
         """
         We test the following:
@@ -474,8 +427,8 @@ class TestBinGlance(functional.FunctionalTest):
 
         line = lines[0]
 
-        image_id, name, disk_format, container_format, size = \
-            [c.strip() for c in line.split()]
+        img_info = [c.strip() for c in line.split()]
+        image_id, name, disk_format, container_format, size = img_info
         self.assertEqual('MyImage', name)
 
         self.assertEqual('3', size,
@@ -531,7 +484,7 @@ class TestBinGlance(functional.FunctionalTest):
         # 1. Add public image
         cmd = minimal_add_command(api_port,
                                   'MyImage',
-                                  'location=http://example.com',
+                                  'location=http://localhost:0',
                                   public=False)
 
         exitcode, out, err = execute(cmd)
@@ -571,8 +524,8 @@ class TestBinGlance(functional.FunctionalTest):
 
         # 5. Update the image's Name attribute
         updated_image_name = "Updated image name"
-        cmd = "bin/glance --port=%d update %s is_public=True name=\"%s\"" \
-                % (api_port, image_id, updated_image_name)
+        cmd = ("bin/glance --port=%d update %s is_public=True name=\"%s\"" %
+               (api_port, image_id, updated_image_name))
 
         exitcode, out, err = execute(cmd)
 
@@ -604,8 +557,12 @@ class TestBinGlance(functional.FunctionalTest):
         self.cleanup()
 
         # Start servers with a Swift backend and a bad auth URL
-        options = {'default_store': 'swift',
-                   'swift_store_auth_address': 'badurl'}
+        override_options = {
+            'default_store': 'swift',
+            'swift_store_auth_address': 'badurl',
+        }
+        options = self.__dict__.copy()
+        options.update(override_options)
         self.start_servers(**options)
 
         api_port = self.api_port
@@ -643,7 +600,6 @@ class TestBinGlance(functional.FunctionalTest):
 
         self.stop_servers()
 
-    @functional.runs_sql
     def test_add_location_with_checksum(self):
         """
         We test the following:
@@ -660,7 +616,7 @@ class TestBinGlance(functional.FunctionalTest):
         # 1. Add public image
         cmd = minimal_add_command(api_port,
                                  'MyImage',
-                                 'location=http://example.com checksum=1')
+                                 'location=http://localhost:0 checksum=1')
         exitcode, out, err = execute(cmd)
 
         self.assertEqual(0, exitcode)
@@ -675,7 +631,6 @@ class TestBinGlance(functional.FunctionalTest):
 
         self.stop_servers()
 
-    @functional.runs_sql
     def test_add_location_without_checksum(self):
         """
         We test the following:
@@ -692,7 +647,7 @@ class TestBinGlance(functional.FunctionalTest):
         # 1. Add public image
         cmd = minimal_add_command(api_port,
                                  'MyImage',
-                                 'location=http://example.com')
+                                 'location=http://localhost:0')
         exitcode, out, err = execute(cmd)
 
         self.assertEqual(0, exitcode)
@@ -707,7 +662,6 @@ class TestBinGlance(functional.FunctionalTest):
 
         self.stop_servers()
 
-    @functional.runs_sql
     def test_add_clear(self):
         """
         We test the following:
@@ -849,8 +803,8 @@ class TestBinGlance(functional.FunctionalTest):
         self.assertEqual(image_lines[0].split()[0], image_ids[1])
 
         # 9. Check past changes-since
-        dt1 = datetime.datetime.utcnow() - datetime.timedelta(1)
-        iso1 = utils.isotime(dt1)
+        dt1 = timeutils.utcnow() - datetime.timedelta(1)
+        iso1 = timeutils.isotime(dt1)
         cmd = "changes-since=%s" % iso1
         exitcode, out, err = execute("%s %s" % (_index_cmd, cmd))
 
@@ -862,8 +816,8 @@ class TestBinGlance(functional.FunctionalTest):
         self.assertEqual(image_lines[2].split()[0], image_ids[0])
 
         # 10. Check future changes-since
-        dt2 = datetime.datetime.utcnow() + datetime.timedelta(1)
-        iso2 = utils.isotime(dt2)
+        dt2 = timeutils.utcnow() + datetime.timedelta(1)
+        iso2 = timeutils.isotime(dt2)
         cmd = "changes-since=%s" % iso2
         exitcode, out, err = execute("%s %s" % (_index_cmd, cmd))
 
@@ -878,9 +832,9 @@ class TestBinGlance(functional.FunctionalTest):
 
         self.assertEqual(0, exitcode)
         image_lines = out.split("\n")[1:-1]
-        self.assertEqual(26, len(image_lines))
+        self.assertEqual(30, len(image_lines))
         self.assertEqual(image_lines[1].split()[1], image_ids[1])
-        self.assertEqual(image_lines[14].split()[1], image_ids[0])
+        self.assertEqual(image_lines[16].split()[1], image_ids[0])
 
         # 12. Check min_ram filter
         cmd = "min_ram=256"
@@ -888,7 +842,7 @@ class TestBinGlance(functional.FunctionalTest):
 
         self.assertEqual(0, exitcode)
         image_lines = out.split("\n")[2:-1]
-        self.assertEqual(12, len(image_lines))
+        self.assertEqual(14, len(image_lines))
         self.assertEqual(image_lines[0].split()[1], image_ids[2])
 
         # 13. Check min_disk filter
@@ -897,7 +851,7 @@ class TestBinGlance(functional.FunctionalTest):
 
         self.assertEqual(0, exitcode)
         image_lines = out.split("\n")[2:-1]
-        self.assertEqual(12, len(image_lines))
+        self.assertEqual(14, len(image_lines))
         self.assertEqual(image_lines[0].split()[1], image_ids[2])
 
         self.stop_servers()
@@ -980,9 +934,9 @@ class TestBinGlance(functional.FunctionalTest):
 
         self.assertEqual(0, exitcode)
         image_lines = out.split("\n")[1:-1]
-        self.assertEqual(24, len(image_lines))
+        self.assertEqual(28, len(image_lines))
         self.assertTrue(image_lines[1].split()[1], image_ids[2])
-        self.assertTrue(image_lines[13].split()[1], image_ids[1])
+        self.assertTrue(image_lines[15].split()[1], image_ids[1])
 
         self.stop_servers()
 
@@ -1056,10 +1010,10 @@ class TestBinGlance(functional.FunctionalTest):
 
         self.assertEqual(0, exitcode)
         image_lines = out.split("\n")[1:-1]
-        self.assertEqual(36, len(image_lines))
+        self.assertEqual(42, len(image_lines))
         self.assertTrue(image_lines[1].split()[1], image_ids[2])
-        self.assertTrue(image_lines[13].split()[1], image_ids[1])
-        self.assertTrue(image_lines[25].split()[1], image_ids[4])
+        self.assertTrue(image_lines[15].split()[1], image_ids[1])
+        self.assertTrue(image_lines[29].split()[1], image_ids[4])
 
         self.stop_servers()
 
@@ -1114,6 +1068,8 @@ class TestBinGlance(functional.FunctionalTest):
         self.assertEqual(0, exitcode)
         self.assertEqual('Deleted image %s' % image_id, out.strip())
 
+        self.stop_servers()
+
     def test_protected_image(self):
         """
         We test the following:
@@ -1164,8 +1120,8 @@ class TestBinGlance(functional.FunctionalTest):
 
         line = lines[0]
 
-        image_id, name, disk_format, container_format, size = \
-            [c.strip() for c in line.split()]
+        img_info = [c.strip() for c in line.split()]
+        image_id, name, disk_format, container_format, size = img_info
         self.assertEqual('MyImage', name)
 
         # 3. Delete the image
@@ -1177,8 +1133,8 @@ class TestBinGlance(functional.FunctionalTest):
         self.assertTrue(out.startswith('You do not have permission'))
 
         # 4. Remove image protection
-        cmd = "bin/glance --port=%d --force update %s" \
-              " protected=False" % (api_port, image_id)
+        cmd = ("bin/glance --port=%d --force update %s "
+               "protected=False" % (api_port, image_id))
 
         exitcode, out, err = execute(cmd)
 
@@ -1202,3 +1158,115 @@ class TestBinGlance(functional.FunctionalTest):
         self.assertEqual('', out.strip())
 
         self.stop_servers()
+
+    def test_timeout(self):
+        self.cleanup()
+
+        keep_sleeping = True
+
+        #start a simple HTTP server in a thread that hangs for a bit
+        class RemoteImageHandler(BaseHTTPServer.BaseHTTPRequestHandler):
+            def do_GET(self):
+                cnt = 1
+                while (keep_sleeping):
+                    cnt += 1
+                    time.sleep(0.1)
+                    if cnt > 100:
+                        break
+
+        server_class = BaseHTTPServer.HTTPServer
+        local_server = server_class(('127.0.0.1', 0), RemoteImageHandler)
+        local_ip, local_port = local_server.server_address
+
+        def serve_requests(httpd):
+            httpd.serve_forever()
+
+        thread.start_new_thread(serve_requests, (local_server,))
+
+        cmd = ("bin/glance --port=%d index --timeout=1") % local_port
+        exitcode, out, err = execute(cmd, raise_error=False)
+
+        keep_sleeping = False
+        local_server.shutdown()
+        self.assertNotEqual(0, exitcode)
+        self.assertTrue("timed out" in out)
+
+    def test_add_member(self):
+        self.cleanup()
+        self.start_servers(**self.__dict__.copy())
+
+        api_port = self.api_port
+        registry_port = self.registry_port
+
+        image_id = "11111111-1111-1111-1111-111111111111"
+        member_id = "21111111-2111-2111-2111-211111111111"
+        member2_id = "31111111-3111-3111-3111-311111111111"
+
+        # 0. Add an image
+        cmd = minimal_add_command(api_port,
+                                  'MyImage',
+                                  'id=%s' % image_id,
+                                  'location=http://example.com')
+        exitcode, out, err = execute(cmd)
+
+        # 1. Add an image member
+        cmd = "bin/glance --port=%d member-add %s %s" % (api_port, image_id,
+                                                         member_id)
+        exitcode, out, err = execute(cmd)
+
+        self.assertEqual(0, exitcode)
+        self.assertEqual('', out.strip())
+
+        # 2. Verify image-members
+        cmd = "bin/glance --port=%d image-members %s " % (api_port, image_id)
+        exitcode, out, err = execute(cmd)
+
+        self.assertEqual(0, exitcode)
+        self.assertTrue(member_id in out)
+
+        # 3. Verify member-images
+        cmd = "bin/glance --port=%d member-images %s " % (api_port, member_id)
+        exitcode, out, err = execute(cmd)
+
+        self.assertEqual(0, exitcode)
+        self.assertTrue(image_id in out)
+
+        # 4. Replace image members
+        cmd = "bin/glance --port=%d members-replace %s %s" % (api_port,
+                                                              image_id,
+                                                              member2_id)
+        exitcode, out, err = execute(cmd)
+
+        self.assertEqual(0, exitcode)
+        self.assertEqual('', out.strip())
+
+        # 5. Verify member-images again for member2
+        cmd = "bin/glance --port=%d member-images %s " % (api_port, member2_id)
+        exitcode, out, err = execute(cmd)
+
+        self.assertEqual(0, exitcode)
+        self.assertTrue(image_id in out)
+
+        # 6. Verify member-images again for member1 (should not be present)
+        cmd = "bin/glance --port=%d member-images %s " % (api_port, member_id)
+        exitcode, out, err = execute(cmd)
+
+        self.assertEqual(0, exitcode)
+        self.assertTrue(image_id not in out)
+
+        # 7. Delete the member
+        cmd = "bin/glance --port=%d member-delete %s %s" % (api_port, image_id,
+                                                            member2_id)
+        exitcode, out, err = execute(cmd)
+
+        self.assertEqual(0, exitcode)
+        self.assertEqual('', out.strip())
+
+        # 8. Verify image-members is empty
+        cmd = "bin/glance --port=%d image-members %s " % (api_port, image_id)
+        exitcode, out, err = execute(cmd)
+
+        self.assertEqual(0, exitcode)
+        self.assertEqual('', out.strip())
+
+        self.stop_servers()
