@@ -1,4 +1,6 @@
-# Copyright 2012 OpenStack Foundation
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
+# Copyright 2012 OpenStack, LLC
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -13,36 +15,35 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import http.client as http
+import json
+
+import requests
 
 from glance.tests import functional
 
 
-class TestSchemas(functional.SynchronousAPIBase):
+class TestSchemas(functional.FunctionalTest):
 
     def setUp(self):
         super(TestSchemas, self).setUp()
-        self.start_server()
+        self.cleanup()
+        self.start_servers(**self.__dict__.copy())
 
     def test_resource(self):
         # Ensure the image link works and custom properties are loaded
-        path = '/v2/schemas/image'
-        response = self.api_get(path)
-        self.assertEqual(http.OK, response.status_code)
-        image_schema = response.json
+        path = 'http://%s:%d/v2/schemas/image' % ('127.0.0.1', self.api_port)
+        response = requests.get(path)
+        self.assertEqual(response.status_code, 200)
+        image_schema = json.loads(response.text)
         expected = set([
             'id',
             'name',
             'visibility',
             'checksum',
-            'os_hash_algo',
-            'os_hash_value',
             'created_at',
             'updated_at',
             'tags',
             'size',
-            'virtual_size',
-            'owner',
             'container_format',
             'disk_format',
             'self',
@@ -50,19 +51,18 @@ class TestSchemas(functional.SynchronousAPIBase):
             'status',
             'schema',
             'direct_url',
-            'locations',
             'min_ram',
             'min_disk',
             'protected',
-            'os_hidden',
-            'stores',
         ])
         self.assertEqual(expected, set(image_schema['properties'].keys()))
 
         # Ensure the images link works and agrees with the image schema
-        path = '/v2/schemas/images'
-        response = self.api_get(path)
-        self.assertEqual(http.OK, response.status_code)
-        images_schema = response.json
+        path = 'http://%s:%d/v2/schemas/images' % ('127.0.0.1', self.api_port)
+        response = requests.get(path)
+        self.assertEqual(response.status_code, 200)
+        images_schema = json.loads(response.text)
         item_schema = images_schema['properties']['images']['items']
         self.assertEqual(item_schema, image_schema)
+
+        self.stop_servers()
