@@ -18,7 +18,6 @@
 """Stubouts, mocks and fixtures for the test suite"""
 
 import os
-import shutil
 
 try:
     import sendfile
@@ -26,13 +25,12 @@ try:
 except ImportError:
     SENDFILE_SUPPORTED = False
 
+import routes
 import webob
 
-from glance.api.middleware import version_negotiation
+from glance.api.middleware import context
 from glance.api.v1 import router
 import glance.common.client
-from glance.common import context
-from glance.common import exception
 from glance.registry.api import v1 as rserver
 from glance.tests import utils
 
@@ -41,6 +39,35 @@ VERBOSE = False
 DEBUG = False
 
 
+class FakeRegistryConnection(object):
+
+    def __init__(self, *args, **kwargs):
+        pass
+
+    def connect(self):
+        return True
+
+    def close(self):
+        return True
+
+    def request(self, method, url, body=None, headers=None):
+        self.req = webob.Request.blank("/" + url.lstrip("/"))
+        self.req.method = method
+        if headers:
+            self.req.headers = headers
+        if body:
+            self.req.body = body
+
+    def getresponse(self):
+        mapper = routes.Mapper()
+        api = context.UnauthenticatedContextMiddleware(rserver.API(mapper))
+        webob_res = self.req.get_response(api)
+
+        return utils.FakeHTTPResponse(status=webob_res.status_int,
+                                      headers=webob_res.headers,
+                                      data=webob_res.body)
+
+
 def stub_out_registry_and_store_server(stubs, base_dir):
     """
     Mocks calls to 127.0.0.1 on 9191 and 9292 for testing so
@@ -48,45 +75,6 @@ def stub_out_registry_and_store_server(stubs, base_dir):
     running
     """
 
-    class FakeRegistryConnection(object):
-
-        def __init__(self, *args, **kwargs):
-            pass
-
-        def connect(self):
-            return True
-
-        def close(self):
-            return True
-
-        def request(self, method, url, body=None, headers={}):
-            self.req = webob.Request.blank("/" + url.lstrip("/"))
-            self.req.method = method
-            if headers:
-                self.req.headers = headers
-            if body:
-                self.req.body = body
-
-        def getresponse(self):
-            sql_connection = os.environ.get('GLANCE_SQL_CONNECTION',
-                                            "sqlite://")
-            context_class = 'glance.registry.context.RequestContext'
-            conf = utils.TestConfigOpts({
-                    'sql_connection': sql_connection,
-                    'verbose': VERBOSE,
-                    'debug': DEBUG
-                    })
-            api = context.ContextMiddleware(rserver.API(conf),
-                                            conf, context_class=context_class)
-            res = self.req.get_response(api)
-
-            # httplib.Response has a read() method...fake it out
-            def fake_reader():
-                return res.body
-
-            setattr(res, 'read', fake_reader)
-            return res
-
     class FakeSocket(object):
 
         def __init__(self, *args, **kwargs):
@@ -110,7 +98,8 @@ def stub_out_registry_and_store_server(stubs, base_dir):
 
         def __init__(self, *args, **kwargs):
             self.sock = FakeSocket()
-            pass
+            self.stub_force_sendfile = kwargs.get('stub_force_sendfile',
+                                                  SENDFILE_SUPPORTED)
 
         def connect(self):
             return True
@@ -118,9 +107,13 @@ def stub_out_registry_and_store_server(stubs, base_dir):
         def close(self):
             return True
 
+        def _clean_url(self, url):
+            #TODO(bcwaldon): Fix the hack that strips off v1
+            return url.replace('/v1', '', 1) if url.startswith('/v1') else url
+
         def putrequest(self, method, url):
-            self.req = webob.Request.blank("/" + url.lstrip("/"))
-            if SENDFILE_SUPPORTED:
+            self.req = webob.Request.blank(self._clean_url(url))
+            if self.stub_force_sendfile:
                 fake_sendfile = FakeSendFile(self.req)
                 stubs.Set(sendfile, 'sendfile', fake_sendfile.sendfile)
             self.req.method = method
@@ -129,7 +122,10 @@ def stub_out_registry_and_store_server(stubs, base_dir):
             self.req.headers[key] = value
 
         def endheaders(self):
-            pass
+            hl = [i.lower() for i in self.req.headers.keys()]
+            assert not ('content-length' in hl and
+                        'transfer-encoding' in hl), \
+                'Content-Length and Transfer-Encoding are mutually exclusive'
 
         def send(self, data):
             # send() is called during chunked-transfer encoding, and
@@ -137,8 +133,8 @@ def stub_out_registry_and_store_server(stubs, base_dir):
             # only write the actual data in tests.
             self.req.body += data.split("\r\n")[1]
 
-        def request(self, method, url, body=None, headers={}):
-            self.req = webob.Request.blank("/" + url.lstrip("/"))
+        def request(self, method, url, body=None, headers=None):
+            self.req = webob.Request.blank(self._clean_url(url))
             self.req.method = method
             if headers:
                 self.req.headers = headers
@@ -146,20 +142,8 @@ def stub_out_registry_and_store_server(stubs, base_dir):
                 self.req.body = body
 
         def getresponse(self):
-            conf = utils.TestConfigOpts({
-                    'verbose': VERBOSE,
-                    'debug': DEBUG,
-                    'bind_host': '0.0.0.0',
-                    'bind_port': '9999999',
-                    'registry_host': '0.0.0.0',
-                    'registry_port': '9191',
-                    'default_store': 'file',
-                    'filesystem_store_datadir': base_dir,
-                    'policy_file': os.path.join(base_dir, 'policy.json'),
-                    })
-            api = version_negotiation.VersionNegotiationFilter(
-                context.ContextMiddleware(router.API(conf), conf),
-                conf)
+            mapper = routes.Mapper()
+            api = context.UnauthenticatedContextMiddleware(router.API(mapper))
             res = self.req.get_response(api)
 
             # httplib.Response has a read() method...fake it out
@@ -187,8 +171,21 @@ def stub_out_registry_and_store_server(stubs, base_dir):
         for i in self.source.app_iter:
             yield i
 
+    def fake_sendable(self, body):
+        force = getattr(self, 'stub_force_sendfile', None)
+        if force is None:
+            return self._stub_orig_sendable(body)
+        else:
+            if force:
+                assert glance.common.client.SENDFILE_SUPPORTED
+            return force
+
     stubs.Set(glance.common.client.BaseClient, 'get_connection_type',
               fake_get_connection_type)
+    setattr(glance.common.client.BaseClient, '_stub_orig_sendable',
+              glance.common.client.BaseClient._sendable)
+    stubs.Set(glance.common.client.BaseClient, '_sendable',
+              fake_sendable)
     stubs.Set(glance.common.client.ImageBodyIterator, '__iter__',
               fake_image_iter)
 
@@ -199,45 +196,6 @@ def stub_out_registry_server(stubs, **kwargs):
     that a real Glance Registry server does not need to be up and
     running
     """
-
-    class FakeRegistryConnection(object):
-
-        def __init__(self, *args, **kwargs):
-            pass
-
-        def connect(self):
-            return True
-
-        def close(self):
-            return True
-
-        def request(self, method, url, body=None, headers={}):
-            self.req = webob.Request.blank("/" + url.lstrip("/"))
-            self.req.method = method
-            if headers:
-                self.req.headers = headers
-            if body:
-                self.req.body = body
-
-        def getresponse(self):
-            sql_connection = kwargs.get('sql_connection', "sqlite:///")
-            context_class = 'glance.registry.context.RequestContext'
-            conf = utils.TestConfigOpts({
-                    'sql_connection': sql_connection,
-                    'verbose': VERBOSE,
-                    'debug': DEBUG
-                    })
-            api = context.ContextMiddleware(rserver.API(conf),
-                                            conf, context_class=context_class)
-            res = self.req.get_response(api)
-
-            # httplib.Response has a read() method...fake it out
-            def fake_reader():
-                return res.body
-
-            setattr(res, 'read', fake_reader)
-            return res
-
     def fake_get_connection_type(client):
         """
         Returns the proper connection type
