@@ -16,37 +16,33 @@
 #    under the License.
 
 import datetime
-import json
 import os
-import StringIO
 import tempfile
-import unittest
-
-import stubout
-import webob
 
 from glance import client
 from glance.common import client as base_client
+from glance.common import config
 from glance.common import exception
 from glance.common import utils
-from glance.registry.db import api as db_api
-from glance.registry.db import models as db_models
+from glance import context
+from glance.db.sqlalchemy import api as db_api
+from glance.db.sqlalchemy import models as db_models
+from glance.openstack.common import timeutils
 from glance.registry import client as rclient
-from glance.registry import context as rcontext
-from glance.tests import stubs
-from glance.tests import utils as test_utils
 from glance.tests.unit import base
 from glance.tests import utils as test_utils
 
-CONF = {'sql_connection': 'sqlite://'}
 
 _gen_uuid = utils.generate_uuid
 
 UUID1 = _gen_uuid()
 UUID2 = _gen_uuid()
 
+#NOTE(bcwaldon): needed to init config_dir cli opt
+config.parse_args()
 
-class TestBadClients(unittest.TestCase):
+
+class TestBadClients(test_utils.BaseTestCase):
 
     """Test exceptions raised for bad clients"""
 
@@ -155,8 +151,8 @@ class TestRegistryClient(base.IsolatedUnitTest):
     def setUp(self):
         """Establish a clean test environment"""
         super(TestRegistryClient, self).setUp()
-        db_api.configure_db(self.conf)
-        self.context = rcontext.RequestContext(is_admin=True)
+        db_api.configure_db()
+        self.context = context.RequestContext(is_admin=True)
         self.FIXTURES = [
             {'id': UUID1,
              'name': 'fake image #1',
@@ -164,8 +160,8 @@ class TestRegistryClient(base.IsolatedUnitTest):
              'disk_format': 'ami',
              'container_format': 'ami',
              'is_public': False,
-             'created_at': datetime.datetime.utcnow(),
-             'updated_at': datetime.datetime.utcnow(),
+             'created_at': timeutils.utcnow(),
+             'updated_at': timeutils.utcnow(),
              'deleted_at': None,
              'deleted': False,
              'checksum': None,
@@ -178,8 +174,8 @@ class TestRegistryClient(base.IsolatedUnitTest):
              'disk_format': 'vhd',
              'container_format': 'ovf',
              'is_public': True,
-             'created_at': datetime.datetime.utcnow(),
-             'updated_at': datetime.datetime.utcnow(),
+             'created_at': timeutils.utcnow(),
+             'updated_at': timeutils.utcnow(),
              'deleted_at': None,
              'deleted': False,
              'checksum': None,
@@ -426,7 +422,7 @@ class TestRegistryClient(base.IsolatedUnitTest):
         Tests that the /images registry API returns list of
         public images sorted by created_at in ascending order.
         """
-        now = datetime.datetime.utcnow()
+        now = timeutils.utcnow()
         time1 = now + datetime.timedelta(seconds=5)
         time2 = now
 
@@ -468,7 +464,7 @@ class TestRegistryClient(base.IsolatedUnitTest):
         Tests that the /images registry API returns list of
         public images sorted by updated_at in descending order.
         """
-        now = datetime.datetime.utcnow()
+        now = timeutils.utcnow()
         time1 = now + datetime.timedelta(seconds=5)
         time2 = now
 
@@ -824,17 +820,17 @@ class TestRegistryClient(base.IsolatedUnitTest):
 
     def test_get_image_details_with_changes_since(self):
         """Tests that a detailed call can be filtered by size_min"""
-        dt1 = datetime.datetime.utcnow() - datetime.timedelta(1)
-        iso1 = utils.isotime(dt1)
+        dt1 = timeutils.utcnow() - datetime.timedelta(1)
+        iso1 = timeutils.isotime(dt1)
 
-        dt2 = datetime.datetime.utcnow() + datetime.timedelta(1)
-        iso2 = utils.isotime(dt2)
+        dt2 = timeutils.utcnow() + datetime.timedelta(1)
+        iso2 = timeutils.isotime(dt2)
 
-        dt3 = datetime.datetime.utcnow() + datetime.timedelta(2)
-        iso3 = utils.isotime(dt3)
+        dt3 = timeutils.utcnow() + datetime.timedelta(2)
+        iso3 = timeutils.isotime(dt3)
 
-        dt4 = datetime.datetime.utcnow() + datetime.timedelta(3)
-        iso4 = utils.isotime(dt4)
+        dt4 = timeutils.utcnow() + datetime.timedelta(3)
+        iso4 = timeutils.isotime(dt4)
 
         UUID3 = _gen_uuid()
         extra_fixture = {'id': UUID3,
@@ -1062,57 +1058,6 @@ class TestRegistryClient(base.IsolatedUnitTest):
                           self.client.add_image,
                           fixture)
 
-    def test_add_image_with_acceptably_long_name(self):
-        """Tests adding image with acceptably long name"""
-        name = 'x' * 255
-        fixture = {'name': name,
-                   'is_public': True,
-                   'disk_format': 'vmdk',
-                   'container_format': 'ovf',
-                   'size': 19,
-                   'location': "file:///tmp/glance-tests/2",
-                  }
-
-        new_image = self.client.add_image(fixture)
-
-        data = self.client.get_image(new_image['id'])
-        self.assertEquals(name, data['name'])
-
-    def test_add_image_with_excessively_long_name(self):
-        """Tests adding image with excessively long name"""
-        name = 'x' * 256
-        fixture = {'name': name,
-                   'is_public': True,
-                   'disk_format': 'vmdk',
-                   'container_format': 'ovf',
-                   'size': 19,
-                   'location': "file:///tmp/glance-tests/2",
-                  }
-
-        self.assertRaises(exception.Invalid,
-                          self.client.add_image,
-                          fixture)
-
-    def test_update_image_with_acceptably_long_name(self):
-        """Tests updating image with acceptably long name"""
-        name = 'x' * 255
-        fixture = {'name': name}
-
-        self.assertTrue(self.client.update_image(UUID2, fixture))
-
-        data = self.client.get_image(UUID2)
-        self.assertEquals(name, data['name'])
-
-    def test_update_image_with_excessively_long_name(self):
-        """Tests updating image with excessively long name"""
-        name = 'x' * 256
-        fixture = {'name': name}
-
-        self.assertRaises(exception.Invalid,
-                          self.client.update_image,
-                          UUID2,
-                          fixture)
-
     def test_update_image(self):
         """Tests that the /images PUT registry API updates the image"""
         fixture = {'name': 'fake public image #2',
@@ -1146,7 +1091,12 @@ class TestRegistryClient(base.IsolatedUnitTest):
         orig_num_images = len(self.client.get_images())
 
         # Delete image #2
-        self.assertTrue(self.client.delete_image(UUID2))
+        image = self.FIXTURES[1]
+        deleted_image = self.client.delete_image(image['id'])
+        self.assertTrue(deleted_image)
+        self.assertEquals(image['id'], deleted_image['id'])
+        self.assertTrue(deleted_image['deleted'])
+        self.assertTrue(deleted_image['deleted_at'])
 
         # Verify one less image
         new_num_images = len(self.client.get_images())
@@ -1166,7 +1116,7 @@ class TestRegistryClient(base.IsolatedUnitTest):
         self.assertEquals(num_members, 0)
 
     def test_get_image_members_not_existing(self):
-        """Tests getting non-existant image members"""
+        """Tests getting non-existent image members"""
         self.assertRaises(exception.NotFound,
                           self.client.get_image_members,
                           _gen_uuid())
@@ -1177,21 +1127,16 @@ class TestRegistryClient(base.IsolatedUnitTest):
         num_members = len(memb_list)
         self.assertEquals(num_members, 0)
 
-    def test_replace_members(self):
+    def test_add_replace_members(self):
         """Tests replacing image members"""
-        self.assertRaises(exception.NotAuthenticated,
-                          self.client.replace_members, UUID2,
-                          dict(member_id='pattieblack'))
+        self.assertTrue(self.client.add_member(UUID2, 'pattieblack'))
+        self.assertTrue(self.client.replace_members(UUID2,
+                          dict(member_id='pattieblack2')))
 
-    def test_add_member(self):
-        """Tests adding image members"""
-        self.assertRaises(exception.NotAuthenticated,
-                          self.client.add_member, UUID2, 'pattieblack')
-
-    def test_delete_member(self):
+    def test_add_delete_member(self):
         """Tests deleting image members"""
-        self.assertRaises(exception.NotAuthenticated,
-                          self.client.delete_member, UUID2, 'pattieblack')
+        self.client.add_member(UUID2, 'pattieblack')
+        self.assertTrue(self.client.delete_member(UUID2, 'pattieblack'))
 
 
 class TestClient(base.IsolatedUnitTest):
@@ -1204,7 +1149,7 @@ class TestClient(base.IsolatedUnitTest):
     def setUp(self):
         """Establish a clean test environment"""
         super(TestClient, self).setUp()
-        db_api.configure_db(self.conf)
+        db_api.configure_db()
         self.client = client.Client("0.0.0.0")
         self.FIXTURES = [
             {'id': UUID1,
@@ -1213,8 +1158,8 @@ class TestClient(base.IsolatedUnitTest):
              'disk_format': 'ami',
              'container_format': 'ami',
              'is_public': False,
-             'created_at': datetime.datetime.utcnow(),
-             'updated_at': datetime.datetime.utcnow(),
+             'created_at': timeutils.utcnow(),
+             'updated_at': timeutils.utcnow(),
              'deleted_at': None,
              'deleted': False,
              'checksum': None,
@@ -1227,15 +1172,15 @@ class TestClient(base.IsolatedUnitTest):
              'disk_format': 'vhd',
              'container_format': 'ovf',
              'is_public': True,
-             'created_at': datetime.datetime.utcnow(),
-             'updated_at': datetime.datetime.utcnow(),
+             'created_at': timeutils.utcnow(),
+             'updated_at': timeutils.utcnow(),
              'deleted_at': None,
              'deleted': False,
              'checksum': None,
              'size': 19,
              'location': "file:///%s/%s" % (self.test_dir, UUID2),
              'properties': {}}]
-        self.context = rcontext.RequestContext(is_admin=True)
+        self.context = context.RequestContext(is_admin=True)
         self.destroy_fixtures()
         self.create_fixtures()
 
@@ -1542,17 +1487,17 @@ class TestClient(base.IsolatedUnitTest):
 
     def test_get_image_details_with_changes_since(self):
         """Tests that a detailed call can be filtered by size_min"""
-        dt1 = datetime.datetime.utcnow() - datetime.timedelta(1)
-        iso1 = utils.isotime(dt1)
+        dt1 = timeutils.utcnow() - datetime.timedelta(1)
+        iso1 = timeutils.isotime(dt1)
 
-        dt2 = datetime.datetime.utcnow() + datetime.timedelta(1)
-        iso2 = utils.isotime(dt2)
+        dt2 = timeutils.utcnow() + datetime.timedelta(1)
+        iso2 = timeutils.isotime(dt2)
 
-        dt3 = datetime.datetime.utcnow() + datetime.timedelta(2)
-        iso3 = utils.isotime(dt3)
+        dt3 = timeutils.utcnow() + datetime.timedelta(2)
+        iso3 = timeutils.isotime(dt3)
 
-        dt4 = datetime.datetime.utcnow() + datetime.timedelta(3)
-        iso4 = utils.isotime(dt4)
+        dt4 = timeutils.utcnow() + datetime.timedelta(3)
+        iso4 = timeutils.isotime(dt4)
 
         UUID3 = _gen_uuid()
         extra_fixture = {'id': UUID3,
@@ -1882,6 +1827,31 @@ class TestClient(base.IsolatedUnitTest):
         for k, v in fixture.items():
             self.assertEquals(v, new_meta[k])
 
+    def test_added_image_notdoubled(self):
+        """Tests contents of an added small seekable image, when using ssl"""
+        fixture = {'name': 'fake public image',
+                   'disk_format': 'vhd',
+                   'container_format': 'ovf'
+                  }
+
+        tmp_fp = tempfile.TemporaryFile('w+')
+        image_data_fixture = _gen_uuid()
+        tmp_fp.write(image_data_fixture)
+        tmp_fp.seek(0)
+
+        self.client.use_ssl = True
+        new_image = self.client.add_image(fixture, tmp_fp)
+        new_image_id = new_image['id']
+
+        tmp_fp.close()
+
+        new_meta, new_image_chunks = self.client.get_image(new_image_id)
+        new_image_data = ""
+        for chunk in new_image_chunks:
+            new_image_data += chunk
+
+        self.assertEquals(image_data_fixture, new_image_data)
+
     @test_utils.skip_if(not base_client.SENDFILE_SUPPORTED,
                         'sendfile not supported')
     def test_add_image_with_image_data_as_file_with_sendfile(self):
@@ -1987,6 +1957,29 @@ class TestClient(base.IsolatedUnitTest):
                           fixture,
                           image_data_fixture)
 
+    def test_add_image_with_unsupported_feature(self):
+        """Tests that UnsupportedHeaderFeature is raised when image is added"""
+        fixture = {
+            'name': 'fake public image',
+            'is_public': True,
+            'disk_format': 'vhd',
+            'container_format': 'ovf',
+            'size': 19,
+            'location': "http://localhost/glance-tests/2"
+        }
+
+        feature_fixture = {
+            'content-type': 'bad content type',
+            'content-length': '0',
+            'x-image-meta-size': '0'
+        }
+
+        for k, v in feature_fixture.items():
+            self.assertRaises(exception.UnsupportedHeaderFeature,
+                              self.client.add_image,
+                              None,
+                              features={k: v})
+
     def test_update_image(self):
         """Tests that the /images PUT registry API updates the image"""
         fixture = {'name': 'fake public image #2',
@@ -2014,6 +2007,25 @@ class TestClient(base.IsolatedUnitTest):
                           _gen_uuid(),
                           fixture)
 
+    def test_update_image_with_unsupported_feature(self):
+        """Tests that UnsupportedHeaderFeature is raised during update"""
+        fixture = {
+            'name': 'fake public image #2'
+        }
+
+        feature_fixture = {
+            'content-type': 'bad content-type',
+            'content-length': '0',
+            'x-image-meta-size': '0'
+        }
+
+        for k, v in feature_fixture.items():
+            self.assertRaises(exception.UnsupportedHeaderFeature,
+                              self.client.update_image,
+                              UUID2,
+                              image_meta=fixture,
+                              features={k: v})
+
     def test_delete_image(self):
         """Tests that image metadata is deleted properly"""
         # Grab the original number of images
@@ -2040,7 +2052,7 @@ class TestClient(base.IsolatedUnitTest):
         self.assertEquals(num_members, 0)
 
     def test_get_image_members_not_existing(self):
-        """Tests getting non-existant image members"""
+        """Tests getting non-existent image members"""
         self.assertRaises(exception.NotFound,
                           self.client.get_image_members,
                           _gen_uuid())
@@ -2051,26 +2063,23 @@ class TestClient(base.IsolatedUnitTest):
         num_members = len(memb_list)
         self.assertEquals(num_members, 0)
 
-    def test_replace_members(self):
+    def test_add_replace_members(self):
         """Tests replacing image members"""
-        self.assertRaises(exception.NotAuthenticated,
-                          self.client.replace_members, UUID2,
-                          dict(member_id='pattieblack'))
+        self.assertTrue(self.client.add_member(UUID2, 'pattieblack'))
+        self.assertTrue(self.client.replace_members(UUID2,
+                          dict(member_id='pattieblack2')))
 
-    def test_add_member(self):
-        """Tests adding image members"""
-        self.assertRaises(exception.NotAuthenticated,
-                          self.client.add_member, UUID2, 'pattieblack')
-
-    def test_delete_member(self):
+    def test_add_delete_member(self):
         """Tests deleting image members"""
-        self.assertRaises(exception.NotAuthenticated,
-                          self.client.delete_member, UUID2, 'pattieblack')
+        self.assertTrue(self.client.add_member(UUID2, 'pattieblack'))
+        self.assertTrue(self.client.delete_member(UUID2, 'pattieblack'))
 
 
-class TestConfigureClientFromURL(unittest.TestCase):
+class TestConfigureClientFromURL(test_utils.BaseTestCase):
+
     def setUp(self):
-        self.client = client.Client("0.0.0.0", doc_root="")
+        super(TestConfigureClientFromURL, self).setUp()
+        self.client = client.Client("0.0.0.0")
 
     def assertConfiguration(self, url, host, port, use_ssl, doc_root):
         self.client.configure_from_url(url)
@@ -2079,13 +2088,22 @@ class TestConfigureClientFromURL(unittest.TestCase):
         self.assertEquals(use_ssl, self.client.use_ssl)
         self.assertEquals(doc_root, self.client.doc_root)
 
+    def test_version_specified_in_url(self):
+        self.assertConfiguration(
+            url='http://www.example.com/v1',
+            host='www.example.com',
+            port=80,
+            use_ssl=False,
+            doc_root='/v1'
+        )
+
     def test_no_port_no_ssl_no_doc_root(self):
         self.assertConfiguration(
             url='http://www.example.com',
             host='www.example.com',
             port=80,
             use_ssl=False,
-            doc_root=''
+            doc_root='/v1'
         )
 
     def test_port_ssl_doc_root(self):
@@ -2094,5 +2112,5 @@ class TestConfigureClientFromURL(unittest.TestCase):
             host='www.example.com',
             port=8000,
             use_ssl=True,
-            doc_root='/prefix/'
+            doc_root='/prefix/v1'
         )
