@@ -27,7 +27,6 @@ if possible.
 import ConfigParser
 import datetime
 import os
-import unittest
 import urlparse
 
 from migrate.versioning.repository import Repository
@@ -35,12 +34,15 @@ from sqlalchemy import *
 from sqlalchemy.pool import NullPool
 
 from glance.common import exception
-from glance.registry.db import models
-import glance.registry.db.migration as migration_api
-from glance.tests.utils import execute
+import glance.db.sqlalchemy.migration as migration_api
+#NOTE(bcwaldon): import this to prevent circular import
+from glance.db.sqlalchemy import api
+from glance.db.sqlalchemy import models
+from glance.openstack.common import cfg
+from glance.tests import utils
 
 
-class TestMigrations(unittest.TestCase):
+class TestMigrations(utils.BaseTestCase):
 
     """Test sqlalchemy-migrate migrations"""
 
@@ -50,13 +52,14 @@ class TestMigrations(unittest.TestCase):
     CONFIG_FILE_PATH = os.environ.get('GLANCE_TEST_MIGRATIONS_CONF',
                                       os.path.join('glance', 'tests', 'unit',
                                                    'test_migrations.conf'))
-    REPOSITORY_PATH = os.path.join('glance', 'registry', 'db', 'migrate_repo')
+    REPOSITORY_PATH = 'glance/db/sqlalchemy/migrate_repo'
     REPOSITORY = Repository(REPOSITORY_PATH)
 
     def __init__(self, *args, **kwargs):
         super(TestMigrations, self).__init__(*args, **kwargs)
 
     def setUp(self):
+        super(TestMigrations, self).setUp()
         # Load test databases from the config file. Only do this
         # once. No need to re-run this on each test...
         if not TestMigrations.TEST_DATABASES:
@@ -82,16 +85,26 @@ class TestMigrations(unittest.TestCase):
         self._reset_databases()
 
     def tearDown(self):
+        super(TestMigrations, self).tearDown()
         # We destroy the test data store between each test case,
         # and recreate it, which ensures that we have no side-effects
         # from the tests
         self._reset_databases()
 
     def _reset_databases(self):
+        def _is_sqlite(conn_string):
+            return conn_string.startswith('sqlite')
+
+        def _is_mysql(conn_string):
+            return conn_string.startswith('mysql')
+
+        def _is_postgresql(conn_string):
+            return conn_string.startswith('postgresql')
+
         for key, engine in self.engines.items():
             conn_string = TestMigrations.TEST_DATABASES[key]
             conn_pieces = urlparse.urlparse(conn_string)
-            if conn_string.startswith('sqlite'):
+            if _is_sqlite(conn_string):
                 # We can just delete the SQLite database, which is
                 # the easiest and cleanest solution
                 db_path = conn_pieces.path.strip('/')
@@ -99,7 +112,7 @@ class TestMigrations(unittest.TestCase):
                     os.unlink(db_path)
                 # No need to recreate the SQLite DB. SQLite will
                 # create it for us if it's not there...
-            elif conn_string.startswith('mysql'):
+            elif _is_mysql(conn_string) or _is_postgresql(conn_string):
                 # We can execute the MySQL client to destroy and re-create
                 # the MYSQL database, which is easier and less error-prone
                 # than using SQLAlchemy to do this via MetaData...trust me.
@@ -112,11 +125,21 @@ class TestMigrations(unittest.TestCase):
                 if len(auth_pieces) > 1:
                     if auth_pieces[1].strip():
                         password = "-p%s" % auth_pieces[1]
-                sql = ("drop database if exists %(database)s; "
-                       "create database %(database)s;") % locals()
-                cmd = ("mysql -u%(user)s %(password)s -h%(host)s "
-                       "-e\"%(sql)s\"") % locals()
-                exitcode, out, err = execute(cmd)
+                if _is_mysql(conn_string):
+                    sql = ("drop database if exists %(database)s; "
+                           "create database %(database)s;") % locals()
+                    cmd = ("mysql -u%(user)s %(password)s -h%(host)s "
+                           "-e\"%(sql)s\"") % locals()
+                if _is_postgresql(conn_string):
+                    cmd = ("dropdb %(database)s ; "
+                           "createdb %(database)s -O %(user)s ; "
+                           "echo \"select 'drop table if exists ' "
+                           "|| tablename || ' cascade;' "
+                           "from pg_tables where schemaname = 'public';\" | "
+                           "psql -d %(database)s | grep '^\s*drop' | "
+                           "psql -d %(database)s"
+                          ) % locals()
+                exitcode, out, err = utils.execute(cmd)
                 self.assertEqual(0, exitcode)
 
     def test_walk_versions(self):
@@ -125,8 +148,8 @@ class TestMigrations(unittest.TestCase):
         that there are no errors in the version scripts for each engine
         """
         for key, engine in self.engines.items():
-            options = {'sql_connection': TestMigrations.TEST_DATABASES[key]}
-            self._walk_versions(options)
+            self.config(sql_connection=TestMigrations.TEST_DATABASES[key])
+            self._walk_versions()
 
     def test_version_control_existing_db(self):
         """
@@ -135,12 +158,9 @@ class TestMigrations(unittest.TestCase):
         without errors.
         """
         for key, engine in self.engines.items():
-            #conf = utils.TestConfigOpts({
-            #        'sql_connection': TestMigrations.TEST_DATABASES[key]})
-            #conf.register_opt(cfg.StrOpt('sql_connection'))
-            options = {'sql_connection': TestMigrations.TEST_DATABASES[key]}
+            self.config(sql_connection=TestMigrations.TEST_DATABASES[key])
             self._create_unversioned_001_db(engine)
-            self._walk_versions(options, initial_version=1)
+            self._walk_versions(initial_version=1)
 
     def _create_unversioned_001_db(self, engine):
         # Create the initial version of the images table
@@ -160,7 +180,7 @@ class TestMigrations(unittest.TestCase):
             Column('deleted', Boolean(), nullable=False, default=False))
         images_001.create()
 
-    def _walk_versions(self, options, initial_version=0):
+    def _walk_versions(self, initial_version=0):
         # Determine latest version script from the repo, then
         # upgrade from 1 through to the latest, with no data
         # in the databases. This just checks that the schema itself
@@ -168,26 +188,25 @@ class TestMigrations(unittest.TestCase):
 
         # Assert we are not under version control...
         self.assertRaises(exception.DatabaseMigrationError,
-                          migration_api.db_version,
-                          options)
+                          migration_api.db_version)
         # Place the database under version control
-        migration_api.version_control(options, version=initial_version)
+        migration_api.version_control(version=initial_version)
 
-        cur_version = migration_api.db_version(options)
+        cur_version = migration_api.db_version()
         self.assertEqual(initial_version, cur_version)
 
         for version in xrange(initial_version + 1,
                               TestMigrations.REPOSITORY.latest + 1):
-            migration_api.db_sync(options, version)
-            cur_version = migration_api.db_version(options)
+            migration_api.db_sync(version)
+            cur_version = migration_api.db_version()
             self.assertEqual(cur_version, version)
 
         # Now walk it back down to 0 from the latest, testing
         # the downgrade paths.
         for version in reversed(
             xrange(0, TestMigrations.REPOSITORY.latest)):
-            migration_api.downgrade(options, version)
-            cur_version = migration_api.db_version(options)
+            migration_api.downgrade(version)
+            cur_version = migration_api.db_version()
             self.assertEqual(cur_version, version)
 
     def test_no_data_loss_2_to_3_to_2(self):
@@ -199,14 +218,14 @@ class TestMigrations(unittest.TestCase):
         the image_properties table back into the base image table.
         """
         for key, engine in self.engines.items():
-            options = {'sql_connection': TestMigrations.TEST_DATABASES[key]}
-            self._no_data_loss_2_to_3_to_2(engine, options)
+            self.config(sql_connection=TestMigrations.TEST_DATABASES[key])
+            self._no_data_loss_2_to_3_to_2(engine)
 
-    def _no_data_loss_2_to_3_to_2(self, engine, options):
-        migration_api.version_control(options, version=0)
-        migration_api.upgrade(options, 2)
+    def _no_data_loss_2_to_3_to_2(self, engine):
+        migration_api.version_control(version=0)
+        migration_api.upgrade(2)
 
-        cur_version = migration_api.db_version(options)
+        cur_version = migration_api.db_version()
         self.assertEquals(2, cur_version)
 
         # We are now on version 2. Check that the images table does
@@ -248,9 +267,9 @@ class TestMigrations(unittest.TestCase):
         # Now let's upgrade to 3. This should move the type column
         # to the image_properties table as type properties.
 
-        migration_api.upgrade(options, 3)
+        migration_api.upgrade(3)
 
-        cur_version = migration_api.db_version(options)
+        cur_version = migration_api.db_version()
         self.assertEquals(3, cur_version)
 
         images_table = Table('images', MetaData(), autoload=True,
@@ -274,7 +293,7 @@ class TestMigrations(unittest.TestCase):
         # Downgrade to 2 and check that the type properties were moved
         # to the main image table
 
-        migration_api.downgrade(options, 2)
+        migration_api.downgrade(2)
 
         images_table = Table('images', MetaData(), autoload=True,
                              autoload_with=engine)
@@ -293,3 +312,61 @@ class TestMigrations(unittest.TestCase):
         last_num_image_properties = conn.execute(sel).scalar()
 
         self.assertEqual(num_image_properties - 2, last_num_image_properties)
+
+    def test_no_data_loss_14_to_15(self):
+        for key, engine in self.engines.items():
+            self.config(sql_connection=TestMigrations.TEST_DATABASES[key])
+            self._check_no_data_loss_14_to_15(engine)
+
+    def _check_no_data_loss_14_to_15(self, engine):
+        """
+        Check that migrating swift location credentials to quoted form
+        and back does not result in data loss.
+        """
+        migration_api.version_control(version=0)
+        migration_api.upgrade(14)
+
+        conn = engine.connect()
+        images_table = Table('images', MetaData(), autoload=True,
+                             autoload_with=engine)
+
+        def get_locations():
+            conn = engine.connect()
+            locations = [x[0] for x in
+                         conn.execute(
+                             select(['location'], from_obj=[images_table]))]
+            conn.close()
+            return locations
+
+        unquoted_locations = [
+            'swift://acct:usr:pass@example.com/container/obj-id',
+            'file://foo',
+            ]
+        quoted_locations = [
+            'swift://acct%3Ausr:pass@example.com/container/obj-id',
+            'file://foo',
+            ]
+
+        # Insert images with an unquoted image location
+        now = datetime.datetime.now()
+        kwargs = dict(
+            deleted=False,
+            created_at=now,
+            updated_at=now,
+            status='active',
+            is_public=True,
+            min_disk=0,
+            min_ram=0,
+            )
+        for i, location in enumerate(unquoted_locations):
+            kwargs.update(location=location, id=i)
+            conn.execute(images_table.insert(), [kwargs])
+        conn.close()
+
+        migration_api.upgrade(15)
+
+        self.assertEqual(get_locations(), quoted_locations)
+
+        migration_api.downgrade(14)
+
+        self.assertEqual(get_locations(), unquoted_locations)
